"""
TrendsTag automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

import requests
import sdkgen
from requests import RequestException
from typing import List

from .trend_collection import TrendCollection

class TrendsTag(sdkgen.TagAbstract):
    def __init__(self, http_client: requests.Session, parser: sdkgen.Parser):
        super().__init__(http_client, parser)


    def get_by_woeid(self, woeid: str) -> TrendCollection:
        """
        The Trends lookup endpoint allow developers to get the Trends for a location, specified using the where-on-earth id (WOEID).
        """
        try:
            path_params = {}
            path_params["woeid"] = woeid

            query_params = {}

            query_struct_names = []

            url = self.parser.url("/2/trends/by/woeid/:woeid", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return TrendCollection.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))





"""
fields automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

from pydantic import BaseModel, Field, GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
class Fields(BaseModel):
    media_fields: Optional[str] = Field(default=None, alias="media.fields")
    place_fields: Optional[str] = Field(default=None, alias="place.fields")
    poll_fields: Optional[str] = Field(default=None, alias="poll.fields")
    tweet_fields: Optional[str] = Field(default=None, alias="tweet.fields")
    user_fields: Optional[str] = Field(default=None, alias="user.fields")
    pass



"""
pagination automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

from pydantic import BaseModel, Field, GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
class Pagination(BaseModel):
    start_time: Optional[str] = Field(default=None, alias="start_time")
    end_time: Optional[str] = Field(default=None, alias="end_time")
    since_id: Optional[str] = Field(default=None, alias="since_id")
    until_id: Optional[str] = Field(default=None, alias="until_id")
    max_results: Optional[int] = Field(default=None, alias="max_results")
    pagination_token: Optional[str] = Field(default=None, alias="pagination_token")
    pass



"""
QuoteTag automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

import requests
import sdkgen
from requests import RequestException
from typing import List

from .fields import Fields
from .tweet_collection import TweetCollection

class QuoteTag(sdkgen.TagAbstract):
    def __init__(self, http_client: requests.Session, parser: sdkgen.Parser):
        super().__init__(http_client, parser)


    def get_all(self, tweet_id: str, exclude: str, expansions: str, max_results: int, pagination_token: str, fields: Fields) -> TweetCollection:
        """
        Returns Quote Tweets for a Tweet specified by the requested Tweet ID.
        """
        try:
            path_params = {}
            path_params["tweet_id"] = tweet_id

            query_params = {}
            query_params["exclude"] = exclude
            query_params["expansions"] = expansions
            query_params["max_results"] = max_results
            query_params["pagination_token"] = pagination_token
            query_params["fields"] = fields

            query_struct_names = []
            query_struct_names.append('fields')

            url = self.parser.url("/2/tweets/:tweet_id/quote_tweets", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return TweetCollection.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))





"""
user_collection automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

from pydantic import BaseModel, Field, GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
from .user import User
class UserCollection(BaseModel):
    data: Optional[List[User]] = Field(default=None, alias="data")
    pass



"""
user automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

from pydantic import BaseModel, Field, GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
class User(BaseModel):
    created_at: Optional[str] = Field(default=None, alias="created_at")
    id: Optional[str] = Field(default=None, alias="id")
    name: Optional[str] = Field(default=None, alias="name")
    pinned_tweet_id: Optional[str] = Field(default=None, alias="pinned_tweet_id")
    username: Optional[str] = Field(default=None, alias="username")
    pass



"""
tweet_entity automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

from pydantic import BaseModel, Field, GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
from .tweet_details import TweetDetails
class TweetEntity(BaseModel):
    data: Optional[TweetDetails] = Field(default=None, alias="data")
    pass



"""
tweet automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

from pydantic import BaseModel, Field, GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
from .tweet_geo import TweetGeo
from .tweet_media import TweetMedia
from .tweet_poll import TweetPoll
from .tweet_reply import TweetReply
class Tweet(BaseModel):
    direct_message_deep_link: Optional[str] = Field(default=None, alias="direct_message_deep_link")
    for_super_followers_only: Optional[bool] = Field(default=None, alias="for_super_followers_only")
    geo: Optional[TweetGeo] = Field(default=None, alias="geo")
    media: Optional[TweetMedia] = Field(default=None, alias="media")
    poll: Optional[TweetPoll] = Field(default=None, alias="poll")
    quote_tweet_id: Optional[str] = Field(default=None, alias="quote_tweet_id")
    reply: Optional[TweetReply] = Field(default=None, alias="reply")
    reply_settings: Optional[str] = Field(default=None, alias="reply_settings")
    text: Optional[str] = Field(default=None, alias="text")
    possibly_sensitive: Optional[bool] = Field(default=None, alias="possibly_sensitive")
    lang: Optional[str] = Field(default=None, alias="lang")
    pass



"""
TweetTag automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

import requests
import sdkgen
from requests import RequestException
from typing import List

from .fields import Fields
from .hide_reply import HideReply
from .hide_reply_response import HideReplyResponse
from .tweet import Tweet
from .tweet_collection import TweetCollection
from .tweet_create_response import TweetCreateResponse
from .tweet_delete_response import TweetDeleteResponse
from .tweet_entity import TweetEntity
from .user_collection import UserCollection

class TweetTag(sdkgen.TagAbstract):
    def __init__(self, http_client: requests.Session, parser: sdkgen.Parser):
        super().__init__(http_client, parser)


    def get_all(self, ids: str, expansions: str, fields: Fields) -> TweetCollection:
        """
        Returns a variety of information about the Tweet specified by the requested ID or list of IDs.
        """
        try:
            path_params = {}

            query_params = {}
            query_params["ids"] = ids
            query_params["expansions"] = expansions
            query_params["fields"] = fields

            query_struct_names = []
            query_struct_names.append('fields')

            url = self.parser.url("/2/tweets", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return TweetCollection.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def get(self, tweet_id: str, expansions: str, fields: Fields) -> TweetEntity:
        """
        Returns a variety of information about a single Tweet specified by the requested ID.
        """
        try:
            path_params = {}
            path_params["tweet_id"] = tweet_id

            query_params = {}
            query_params["expansions"] = expansions
            query_params["fields"] = fields

            query_struct_names = []
            query_struct_names.append('fields')

            url = self.parser.url("/2/tweets/:tweet_id", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return TweetEntity.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def create(self, payload: Tweet) -> TweetCreateResponse:
        """
        Creates a Tweet on behalf of an authenticated user.
        """
        try:
            path_params = {}

            query_params = {}

            query_struct_names = []

            url = self.parser.url("/2/tweets", path_params)

            headers = {}
            headers["Content-Type"] = "application/json"

            response = self.http_client.post(url, headers=headers, params=self.parser.query(query_params, query_struct_names), json=payload.model_dump(by_alias=True))

            if response.status_code >= 200 and response.status_code < 300:
                return TweetCreateResponse.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def delete(self, tweet_id: str) -> TweetDeleteResponse:
        """
        Allows a user or authenticated user ID to delete a Tweet.
        """
        try:
            path_params = {}
            path_params["tweet_id"] = tweet_id

            query_params = {}

            query_struct_names = []

            url = self.parser.url("/2/tweets/:tweet_id", path_params)

            headers = {}

            response = self.http_client.delete(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return TweetDeleteResponse.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def hide_reply(self, tweet_id: str, payload: HideReply) -> HideReplyResponse:
        """
        Hides or unhides a reply to a Tweet.
        """
        try:
            path_params = {}
            path_params["tweet_id"] = tweet_id

            query_params = {}

            query_struct_names = []

            url = self.parser.url("/2/tweets/:tweet_id/hidden", path_params)

            headers = {}
            headers["Content-Type"] = "application/json"

            response = self.http_client.put(url, headers=headers, params=self.parser.query(query_params, query_struct_names), json=payload.model_dump(by_alias=True))

            if response.status_code >= 200 and response.status_code < 300:
                return HideReplyResponse.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def get_liking_users(self, tweet_id: str, expansions: str, max_results: int, pagination_token: str) -> UserCollection:
        """
        Allows you to get information about a Tweet’s liking users.
        """
        try:
            path_params = {}
            path_params["tweet_id"] = tweet_id

            query_params = {}
            query_params["expansions"] = expansions
            query_params["max_results"] = max_results
            query_params["pagination_token"] = pagination_token

            query_struct_names = []

            url = self.parser.url("/2/tweets/:tweet_id/liking_users", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return UserCollection.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))





"""
tweet_usage automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

from pydantic import BaseModel, Field, GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
class TweetUsage(BaseModel):
    cap_reset_day: Optional[int] = Field(default=None, alias="cap_reset_day")
    project_cap: Optional[str] = Field(default=None, alias="project_cap")
    project_id: Optional[str] = Field(default=None, alias="project_id")
    project_usage: Optional[str] = Field(default=None, alias="project_usage")
    pass



"""
UserTag automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

import requests
import sdkgen
from requests import RequestException
from typing import List

from .fields import Fields
from .like_response import LikeResponse
from .pagination import Pagination
from .single_tweet import SingleTweet
from .tweet_collection import TweetCollection
from .user import User
from .user_collection import UserCollection

class UserTag(sdkgen.TagAbstract):
    def __init__(self, http_client: requests.Session, parser: sdkgen.Parser):
        super().__init__(http_client, parser)


    def get_all(self, ids: str, expansions: str, fields: Fields) -> UserCollection:
        """
        Returns a variety of information about one or more users specified by the requested IDs.
        """
        try:
            path_params = {}

            query_params = {}
            query_params["ids"] = ids
            query_params["expansions"] = expansions
            query_params["fields"] = fields

            query_struct_names = []
            query_struct_names.append('fields')

            url = self.parser.url("/2/users", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return UserCollection.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def get(self, user_id: str, expansions: str, fields: Fields) -> User:
        """
        Returns a variety of information about a single user specified by the requested ID.
        """
        try:
            path_params = {}
            path_params["user_id"] = user_id

            query_params = {}
            query_params["expansions"] = expansions
            query_params["fields"] = fields

            query_struct_names = []
            query_struct_names.append('fields')

            url = self.parser.url("/2/users/:user_id", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return User.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def get_timeline(self, user_id: str, exclude: str, expansions: str, pagination: Pagination, fields: Fields) -> TweetCollection:
        """
        Allows you to retrieve a collection of the most recent Tweets and Retweets posted by you and users you follow. This endpoint can return every Tweet created on a timeline over the last 7 days as well as the most recent 800 regardless of creation date.
        """
        try:
            path_params = {}
            path_params["user_id"] = user_id

            query_params = {}
            query_params["exclude"] = exclude
            query_params["expansions"] = expansions
            query_params["pagination"] = pagination
            query_params["fields"] = fields

            query_struct_names = []
            query_struct_names.append('pagination')
            query_struct_names.append('fields')

            url = self.parser.url("/2/users/:user_id/timelines/reverse_chronological", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return TweetCollection.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def get_liked_tweets(self, user_id: str, expansions: str, max_results: int, pagination_token: str, fields: Fields) -> TweetCollection:
        """
        Tweets liked by a user
        """
        try:
            path_params = {}
            path_params["user_id"] = user_id

            query_params = {}
            query_params["expansions"] = expansions
            query_params["max_results"] = max_results
            query_params["pagination_token"] = pagination_token
            query_params["fields"] = fields

            query_struct_names = []
            query_struct_names.append('fields')

            url = self.parser.url("/2/users/:user_id/liked_tweets", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return TweetCollection.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def remove_like(self, user_id: str, tweet_id: str) -> LikeResponse:
        """
        Allows a user or authenticated user ID to unlike a Tweet.
        """
        try:
            path_params = {}
            path_params["user_id"] = user_id
            path_params["tweet_id"] = tweet_id

            query_params = {}

            query_struct_names = []

            url = self.parser.url("/2/users/:user_id/likes/:tweet_id", path_params)

            headers = {}

            response = self.http_client.delete(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return LikeResponse.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def create_like(self, user_id: str, payload: SingleTweet) -> LikeResponse:
        """
        Causes the user ID identified in the path parameter to Like the target Tweet.
        """
        try:
            path_params = {}
            path_params["user_id"] = user_id

            query_params = {}

            query_struct_names = []

            url = self.parser.url("/2/users/:user_id/likes", path_params)

            headers = {}
            headers["Content-Type"] = "application/json"

            response = self.http_client.post(url, headers=headers, params=self.parser.query(query_params, query_struct_names), json=payload.model_dump(by_alias=True))

            if response.status_code >= 200 and response.status_code < 300:
                return LikeResponse.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))

    def find_by_name(self, usernames: str, expansions: str, fields: Fields) -> UserCollection:
        """
        Returns a variety of information about one or more users specified by their usernames.
        """
        try:
            path_params = {}

            query_params = {}
            query_params["usernames"] = usernames
            query_params["expansions"] = expansions
            query_params["fields"] = fields

            query_struct_names = []
            query_struct_names.append('fields')

            url = self.parser.url("/2/users/by", path_params)

            headers = {}

            response = self.http_client.get(url, headers=headers, params=self.parser.query(query_params, query_struct_names))

            if response.status_code >= 200 and response.status_code < 300:
                return UserCollection.model_validate_json(json_data=response.content)


            raise sdkgen.UnknownStatusCodeException("The server returned an unknown status code")
        except RequestException as e:
            raise sdkgen.ClientException("An unknown error occurred: " + str(e))





"""
tweet_details automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

from pydantic import BaseModel, Field, GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
import datetime
class TweetDetails(BaseModel):
    id: Optional[str] = Field(default=None, alias="id")
    text: Optional[str] = Field(default=None, alias="text")
    created_at: Optional[datetime.datetime] = Field(default=None, alias="created_at")
    author_id: Optional[str] = Field(default=None, alias="author_id")
    edit_history_tweet_ids: Optional[List[str]] = Field(default=None, alias="edit_history_tweet_ids")
    conversation_id: Optional[str] = Field(default=None, alias="conversation_id")
    in_reply_to_user_id: Optional[str] = Field(default=None, alias="in_reply_to_user_id")
    pass



"""
Client automatically generated by SDKgen please do not edit this file manually
https://sdkgen.app
"""

import requests
import sdkgen
from requests import RequestException
from typing import List

from .tweet_tag import TweetTag
from .usage_tag import UsageTag
from .user_tag import UserTag
from .bookmark_tag import BookmarkTag
from .search_tag import SearchTag
from .quote_tag import QuoteTag
from .trends_tag import TrendsTag

class Client(sdkgen.ClientAbstract):
    def __init__(self, base_url: str, credentials: sdkgen.CredentialsInterface):
        super().__init__(base_url, credentials)

    def tweet(self) -> TweetTag:
        return TweetTag(
            self.http_client,
            self.parser
        )

    def usage(self) -> UsageTag:
        return UsageTag(
            self.http_client,
            self.parser
        )

    def user(self) -> UserTag:
        return UserTag(
            self.http_client,
            self.parser
        )

    def bookmark(self) -> BookmarkTag:
        return BookmarkTag(
            self.http_client,
            self.parser
        )

    def search(self) -> SearchTag:
        return SearchTag(
            self.http_client,
            self.parser
        )

    def quote(self) -> QuoteTag:
        return QuoteTag(
            self.http_client,
            self.parser
        )

    def trends(self) -> TrendsTag:
        return TrendsTag(
            self.http_client,
            self.parser
        )



    @staticmethod
    def build(token: str):
        return Client("https://api.twitter.com", sdkgen.HttpBearer(token))

